@@ -129,12 +129,6 @@ gail_get_accessible_for_widget (GtkWidget *widget,
           g_object_unref (obj);
         }
     }
-  else if (GTK_IS_TREE_VIEW (widget)) 
-    {
-      obj = gail_tree_view_ref_focus_cell (GTK_TREE_VIEW (widget));
-      if (obj)
-        *transient = TRUE;
-    }
   else if (GNOME_IS_CANVAS (widget)) 
     {
       GnomeCanvas *canvas;
@@ -246,6 +240,14 @@ gail_focus_watcher (GSignalInvocationHint *ihint,
           return TRUE;
         }
     }
+  /*
+   * If the focus widget is a GtkSocket without a plug
+   * then ignore the focus notification as the embedded
+   * plug will report a focus notification.
+   */
+  if (GTK_IS_SOCKET (widget) &&
+      GTK_SOCKET (widget)->plug_widget == NULL)
+    return TRUE;
   /*
    * The widget may not yet be visible on the screen so we wait until it is.
    */
@@ -704,7 +706,7 @@ gnome_accessibility_module_shutdown (void)
     }
   gail_initialized = FALSE;
 
-  fprintf (stderr, "Gtk Accessibilty Module shutdown\n");
+  fprintf (stderr, "GTK Accessibility Module shutdown\n");
 
   /* FIXME: de-register the factory types so we can unload ? */
 }
