@@ -22,7 +22,6 @@
 #include "gailclist.h"
 #include "gailclistcell.h"
 #include "gailcellparent.h"
-#include "gailclistfactory.h"
 
 /* Copied from gtkclist.c */
 /* this defigns the base grid spacing */
@@ -952,6 +951,7 @@ static gboolean
 gail_clist_is_row_selected (AtkTable      *table,
                             gint          row)
 {
+  GList *elem;
   GtkWidget *widget;
   GtkCList *clist;
   GtkCListRow *clist_row;
@@ -966,7 +966,10 @@ gail_clist_is_row_selected (AtkTable      *table,
   if (row < 0 || row >= clist->rows)
     return FALSE;
 
-  clist_row = ROW_ELEMENT (clist, row)->data;
+  elem = ROW_ELEMENT (clist, row);
+  if (!elem)
+    return FALSE;
+  clist_row = elem->data;
 
   return (clist_row->state == GTK_STATE_SELECTED);
 }
@@ -1345,9 +1348,14 @@ gail_clist_set_row_data (AtkTable      *table,
     } 
   if (!found)
     {
+      GList *elem;
+
+      elem = ROW_ELEMENT (gtk_clist, row);
+      g_return_if_fail (elem != NULL);
+
       row_data = g_new (GailCListRow, 1);
       row_data->row_number = row;
-      row_data->row_data = ROW_ELEMENT (gtk_clist, row)->data;
+      row_data->row_data = elem->data;
       if (is_header)
         {
           row_data->header = header;
@@ -1452,12 +1460,15 @@ gail_clist_cell_data_new (GailCList     *clist,
                           gint          column,
                           gint          row)
 {
+  GList *elem;
   GailCListCellData *cell_data;
   GtkCList *gtk_clist;
   GtkCListRow *clist_row;
 
   gtk_clist = GTK_CLIST (GTK_ACCESSIBLE (clist)->widget);
-  clist_row = (GtkCListRow *) g_list_nth (gtk_clist->row_list, row)->data;
+  elem = g_list_nth (gtk_clist->row_list, row);
+  g_return_if_fail (elem != NULL);
+  clist_row = (GtkCListRow *) elem->data;
   cell_data = g_new (GailCListCellData, 1);
   cell_data->gail_cell = cell;
   cell_data->gtk_cell = &(clist_row->cell[column]);
