@@ -33,6 +33,7 @@ static G_CONST_RETURN gchar* gail_widget_get_name (AtkObject *accessible);
 static G_CONST_RETURN gchar* gail_widget_get_description (AtkObject *accessible);
 static AtkObject* gail_widget_get_parent (AtkObject *accessible);
 static AtkStateSet* gail_widget_ref_state_set (AtkObject *accessible);
+static AtkRelationSet* gail_widget_ref_relation_set (AtkObject *accessible);
 static gint gail_widget_get_index_in_parent (AtkObject *accessible);
 
 static void atk_component_interface_init (AtkComponentIface *iface);
@@ -158,6 +159,7 @@ gail_widget_class_init (GailWidgetClass *klass)
   class->get_name = gail_widget_get_name;
   class->get_description = gail_widget_get_description;
   class->get_parent = gail_widget_get_parent;
+  class->ref_relation_set = gail_widget_ref_relation_set;
   class->ref_state_set = gail_widget_ref_state_set;
   class->get_index_in_parent = gail_widget_get_index_in_parent;
   class->initialize = gail_widget_real_initialize;
@@ -341,6 +343,114 @@ gail_widget_get_parent (AtkObject *accessible)
   return parent;
 }
 
+static GtkWidget*
+find_label (GtkWidget *start,
+            GtkWidget *widget)
+{
+
+  GList *list, *l;
+  GtkWidget *parent;
+  GtkWidget *label;
+
+  label = NULL;
+  parent = gtk_widget_get_parent (start);
+  if (GTK_IS_CONTAINER (parent))
+    {
+      list = gtk_container_get_children (GTK_CONTAINER (parent));
+
+      l = list;
+      if (l)
+        {
+          while (l)
+            {
+              if ((GtkWidget *)l->data == start)
+                break;
+
+              l = l->next;
+            }
+          /*
+           * The widget may claim container as parent but it may not
+           * be listed as a child.
+           */
+          if (!l)
+            {
+              if (GTK_IS_SCROLLED_WINDOW (parent) && GTK_IS_SCROLLBAR (start))
+                return NULL;
+            }
+          g_return_val_if_fail (l, NULL);
+          if (l->prev && GTK_IS_LABEL (l->prev->data))
+	    {  
+              label = GTK_WIDGET (l->prev->data);
+              if (gtk_label_get_mnemonic_widget (GTK_LABEL (label)) != widget)
+                label = NULL;
+            } 
+          if (!label && l->next && GTK_IS_LABEL (l->next->data))
+	    {  
+              label = GTK_WIDGET (l->next->data);
+              if (gtk_label_get_mnemonic_widget (GTK_LABEL (label)) != widget)
+                label = NULL;
+            } 
+
+          g_list_free (list);
+       }
+    }
+  return label;
+}
+
+static AtkRelationSet*
+gail_widget_ref_relation_set (AtkObject *obj)
+{
+  GtkWidget *widget;
+  AtkRelationSet *relation_set;
+  GtkWidget *label;
+  AtkObject *array[1];
+  AtkRelation* relation;
+
+  g_return_val_if_fail (GAIL_IS_WIDGET (obj), NULL);
+
+  widget = GTK_ACCESSIBLE (obj)->widget;
+  if (widget == NULL)
+    /*
+     * State is defunct
+     */
+    return NULL;
+
+  relation_set = ATK_OBJECT_CLASS (parent_class)->ref_relation_set (obj);
+
+  if (!atk_relation_set_contains (relation_set, ATK_RELATION_LABELLED_BY))
+    {
+      /*
+       * Heuristic to create LABELLED_BY relation:
+       *
+       * If the previous or next child of the current widget is a label 
+       * and its mnemonic widget is the current widget then set LABELLED_BY 
+       * relation on current accessible object.
+       *
+       * If this fails try to find a label at the same level as the widget's
+       * parent. This finds labels for NautilusProgramChooser dialog.
+       */
+      label = find_label (widget, widget);
+      if (!label) 
+        {
+          GtkWidget *parent;
+
+          parent = gtk_widget_get_parent (widget);
+          if (parent)
+            label = find_label (parent, widget);
+        }
+      if (label)
+        {
+	  array [0] = gtk_widget_get_accessible (label);
+
+	  relation = atk_relation_new (array, 1, ATK_RELATION_LABELLED_BY);
+	  atk_relation_set_add (relation_set, relation);
+	  g_object_unref (relation);
+        }
+    }
+
+  return relation_set;
+}
+
 static AtkStateSet*
 gail_widget_ref_state_set (AtkObject *accessible)
 {
@@ -920,7 +1030,27 @@ static gboolean gail_widget_on_screen (GtkWidget *widget)
         return_value = TRUE;
     }
   else
-    return_value = TRUE;
+    {
+      /*
+       * Check whether the widget has been placed of the screen. The
+       * widget may be MAPPED as when toolbar items do not fit on the toolbar.
+       */
+      if (widget->allocation.x + widget->allocation.width <= 0 &&
+          widget->allocation.y + widget->allocation.height <= 0)
+        return_value = FALSE;
+      else 
+        {
+        /*
+         * Check whether the widget has been placed of the screen. The
+         * widget may be MAPPED as when toolbar items do not fit on the toolbar.
+         */
+          if (widget->allocation.x + widget->allocation.width <= 0 &&
+              widget->allocation.y + widget->allocation.height <= 0)
+            return_value = FALSE;
+          else 
+            return_value = TRUE;
+        }
+    }
 
   return return_value;
 }
